print("import: 'aws_xray_sdk'")
import aws_xray_sdk

print("import: 'aws_xray_sdk.core'")
import aws_xray_sdk.core

print("import: 'aws_xray_sdk.core.emitters'")
import aws_xray_sdk.core.emitters

print("import: 'aws_xray_sdk.core.exceptions'")
import aws_xray_sdk.core.exceptions

print("import: 'aws_xray_sdk.core.models'")
import aws_xray_sdk.core.models

print("import: 'aws_xray_sdk.core.plugins'")
import aws_xray_sdk.core.plugins

print("import: 'aws_xray_sdk.core.sampling'")
import aws_xray_sdk.core.sampling

print("import: 'aws_xray_sdk.core.utils'")
import aws_xray_sdk.core.utils

